/* 
 * DNet eBusiness Suite
 * Copyright: Nan21 Electronics SRL. All rights reserved.
 * Use is subject to license terms.
 */

package net.nan21.dnet.module.pj.md.domain.entity;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.QueryHint;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;
import javax.validation.constraints.NotNull;
import net.nan21.dnet.core.domain.eventhandler.DefaultEventHandler;
import net.nan21.dnet.core.domain.model.AbstractAuditable;
import net.nan21.dnet.module.ad.usr.domain.entity.Assignable;
import net.nan21.dnet.module.pj.base.domain.entity.ProjectRole;
import net.nan21.dnet.module.pj.md.domain.entity.Project;
import org.eclipse.persistence.annotations.Customizer;
import org.eclipse.persistence.config.HintValues;
import org.eclipse.persistence.config.QueryHints;
import org.eclipse.persistence.descriptors.DescriptorEvent;

@NamedQueries({
	@NamedQuery(
		name=ProjectMember.NQ_FIND_BY_ID,
		query="SELECT e FROM ProjectMember e WHERE e.clientId = :pClientId and e.id = :pId ",
		hints=@QueryHint(name=QueryHints.BIND_PARAMETERS, value=HintValues.TRUE)
	)
	,@NamedQuery(
		name=ProjectMember.NQ_FIND_BY_IDS,
		query="SELECT e FROM ProjectMember e WHERE e.clientId = :pClientId and e.id in :pIds",
		hints=@QueryHint(name=QueryHints.BIND_PARAMETERS, value=HintValues.TRUE)
	)
	,@NamedQuery(
		name=ProjectMember.NQ_FIND_BY_NAME,
		query="SELECT e FROM ProjectMember e WHERE e.clientId = :pClientId and e.project = :pProject and e.member = :pMember",
		hints=@QueryHint(name=QueryHints.BIND_PARAMETERS, value=HintValues.TRUE)
	)
	,@NamedQuery(
		name=ProjectMember.NQ_FIND_BY_NAME_PRIMITIVE,
		query="SELECT e FROM ProjectMember e WHERE e.clientId = :pClientId and e.project.id = :pProjectId and e.member.id = :pMemberId",
		hints=@QueryHint(name=QueryHints.BIND_PARAMETERS, value=HintValues.TRUE)
	)
})
@Entity
@Table(
	name=ProjectMember.TABLE_NAME
	,uniqueConstraints={
		@UniqueConstraint( 
			name=ProjectMember.TABLE_NAME+"_UK1"
			,columnNames={"CLIENTID","PROJECT_ID","MEMBER_ID"}
		)
	}
)
@Customizer(DefaultEventHandler.class)
public class ProjectMember extends AbstractAuditable  {
	
	public static final String TABLE_NAME = "PJ_PRJ_MBR";
	public static final String SEQUENCE_NAME = "PJ_PRJ_MBR_SEQ";
	
	private static final long serialVersionUID = -8865917134914502125L;
	
	/**
	 * Named query find by ID.
	 */ 
	public static final String NQ_FIND_BY_ID = "ProjectMember.findById";
	
	/**
	 * Named query find by IDs.
	 */     
	public static final String NQ_FIND_BY_IDS = "ProjectMember.findByIds";
	
	/**
	 * Named query find by unique key: Name.
	 */
	public static final String NQ_FIND_BY_NAME = "ProjectMember.findByName";
	
	/**
	 * Named query find by unique key: Name using the ID field for references.
	 */
	public static final String NQ_FIND_BY_NAME_PRIMITIVE = "ProjectMember.findByName_PRIMITIVE";
	
	/**
			 * System generated unique identifier.
			 */
	@Column(name="ID", nullable=false)
	@NotNull
	@Id
	@GeneratedValue(generator=SEQUENCE_NAME)
	private Long id;
	
	@ManyToOne(fetch=FetchType.LAZY, targetEntity=Project.class)
	@JoinColumn(name="PROJECT_ID", referencedColumnName="ID")
	private Project project;
	
	@ManyToOne(fetch=FetchType.LAZY, targetEntity=Assignable.class)
	@JoinColumn(name="MEMBER_ID", referencedColumnName="ID")
	private Assignable member;
	
	@ManyToOne(fetch=FetchType.LAZY, targetEntity=ProjectRole.class)
	@JoinColumn(name="PROJECTROLE_ID", referencedColumnName="ID")
	private ProjectRole projectRole;
	
	public Long getId() {
		return this.id;
	}
	
	public void setId(Long id) {
		this.id = id;
	}
	
	public Project getProject() {
		return this.project;
	}
	
	public void setProject(Project project) {
		if (project != null ) {
			this.__validate_client_context__(project.getClientId());
		}
		this.project = project;
	}
	
	public Assignable getMember() {
		return this.member;
	}
	
	public void setMember(Assignable member) {
		if (member != null ) {
			this.__validate_client_context__(member.getClientId());
		}
		this.member = member;
	}
	
	public ProjectRole getProjectRole() {
		return this.projectRole;
	}
	
	public void setProjectRole(ProjectRole projectRole) {
		if (projectRole != null ) {
			this.__validate_client_context__(projectRole.getClientId());
		}
		this.projectRole = projectRole;
	}
	
	public void aboutToInsert(DescriptorEvent event) {
		super.aboutToInsert(event);
	
	}
}
